<?php

/**
 * Playa
 *
 * This extension adds a one-to-many relationship custom field type.
 *
 * It was based on Multi Relation by Mark Huot.
 *
 * @package   Playa
 * @author    Brandon Kelly <me@brandon-kelly.com>
 * @link      http://brandon-kelly.com/apps/playa
 * @see       http://expressionengine.com/forums/viewthread/39595/
 * @see       http://expressionengine.com/forums/viewthread/78049/
 * @copyright Copyright (c) 2008 Brandon Kelly
 * @license   http://creativecommons.org/licenses/by-sa/3.0/   Attribution-Share Alike 3.0 Unported
 */
class Playa
{
	/**
	 * Extension Settings
	 *
	 * @var array
	 */
	var $settings = array();

	/**
	 * Extension Name
	 *
	 * @var string
	 */
	var $name = 'Playa';
	
	/**
	 * Extension Class Name
	 *
	 * @var string
	 */
	var $class_name = 'Playa';
	
	/**
	 * Extension Field Type
	 *
	 * @var string
	 */
	var $type = 'playa';
	
	/**
	 * Extension Version
	 *
	 * @var string
	 */
	var $version = '1.3.3';
	
	/**
	 * Extension Description
	 *
	 * @var string
	 */
	var $description = 'The proverbial multiple relationships field';
	
	/**
	 * Extension Settings Exist
	 *
	 * If set to 'y', a settings page will be shown in the Extensions Manager
	 *
	 * @var string
	 */
	var $settings_exist = 'y';
	
	/**
	 * Documentation URL
	 *
	 * @var string
	 */
	var $docs_url = 'http://brandon-kelly.com/apps/playa?utm_campaign=playa_em';




	/**
	 * Extension Constructor
	 *
	 * @param array   $settings
	 * @since version 1.0.0
	 */
	function Playa($settings=array())
	{
		$this->settings = $this->get_site_settings($settings);
	}



	/**
	 * Get All Settings
	 *
	 * @return array   All extension settings
	 * @since  version 1.2.1
	 */
	function get_all_settings()
	{
		global $DB;
		
		$query = $DB->query("SELECT settings
		                     FROM exp_extensions
		                     WHERE class = '{$this->class_name}'
		                       AND settings != ''
		                     LIMIT 1");
		
		return $query->num_rows
			? unserialize($query->row['settings'])
			: array();
	}



	/**
	 * Get Site Settings
	 *
	 * @param  array   $settings   Current extension settings (not site-specific)
	 * @return array               Site-specific extension settings
	 * @since  version 1.2.1
	 */
	function get_site_settings($settings=array())
	{
		global $PREFS;
		
		$site_id = $PREFS->ini('site_id');
		return isset($settings[$site_id])
			? $settings[$site_id]
			: array();
	}



	/**
	 * Settings Form
	 *
	 * Construct the custom settings form.
	 *
	 * Look and feel based on LG Addon Updater's settings form.
	 *
	 * @param  array   $current   Current extension settings (not site-specific)
	 * @see    http://expressionengine.com/docs/development/extensions.html#settings
	 * @author Leevi Graham <http://leevigraham.com>
	 * @since  version 1.2.0
	 */
	function settings_form($current)
	{
		global $DB, $DSP, $LANG, $IN, $PREFS;
	    
		$current = $this->get_site_settings($current);

		// Is MSM enabled?
		$msm = ($PREFS->ini('multiple_sites_enabled') == 'y') ? TRUE : FALSE;

		// Get the current Site ID
		$site_id = $PREFS->ini('site_id');
	    
	    // Get Statuses
	    
		$statuses = array();
		$query = $DB->query('SELECT g.group_id, sites.site_label, g.group_name, s.status_id, s.status
		                     FROM exp_status_groups AS g,
			                      exp_statuses      AS s,
			                      exp_sites         AS sites
			                 WHERE sites.site_id = g.site_id
			                   AND s.group_id = g.group_id '
			              . ($msm ? '' : "AND g.site_id = '{$site_id}' ")
			              . 'ORDER BY sites.site_label, g.group_name, s.status_order');

		if ($query->num_rows)
		{
			foreach($query->result as $row)
			{
				if ( ! isset($statuses[$row['group_id']]))
				{
					$statuses[$row['group_id']] = array(
						'group_name' => ($msm ? $row['site_label'].' &ndash; ' : '') . $row['group_name'],
						'statuses'   => array()
					);
				}
				
				$statuses[$row['group_id']]['statuses'][$row['status_id']] = $row['status'];
			}
		}
		
	    
	    // Form Header
	    
	    $DSP->crumbline = TRUE;
	    
	    $DSP->title  = $LANG->line('extension_settings');
	    $DSP->crumb  = $DSP->anchor(BASE.AMP.'C=admin'.AMP.'area=utilities', $LANG->line('utilities')).
	                               $DSP->crumb_item($DSP->anchor(BASE.AMP.'C=admin'.AMP.'M=utilities'.AMP.'P=extensions_manager', $LANG->line('extensions_manager')));
	    $DSP->crumb .= $DSP->crumb_item($this->name);
	    
	    $DSP->right_crumb($LANG->line('disable_extension'), BASE.AMP.'C=admin'.AMP.'M=utilities'.AMP.'P=toggle_extension_confirm'.AMP.'which=disable'.AMP.'name='.$IN->GBL('name'));
	    
	    $DSP->body = '';

		// Donations button
	    $DSP->body .= '<div style="float:right;">'
	                . '<a style="display:block; margin:-2px 10px 0 0; padding:5px 0 5px 70px; width:190px; height:15px; font-size:12px; line-height:15px;'
	                . ' background:url(http://brandon-kelly.com/images/shared/donations.png) no-repeat 0 0; color:#000; font-weight:bold;"'
	                . ' href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=2181794" target="_blank">'
	                . $LANG->line('donate')
	                . '</a>'
	                . '</div>';
	    
	    $DSP->body .= "<h1>{$this->name} <small>{$this->version}</small></h1>";
	    
	    $DSP->body .= $DSP->form_open(
	                                array(
	                                        'action' => 'C=admin'.AMP.'M=utilities'.AMP.'P=save_extension_settings',
	                                        'name'   => 'settings_example',
	                                        'id'     => 'settings_example'
	                                    ),
	                                array('name' => strtolower($this->class_name))
	                            );
		
		
		// Open Statuses Setting
		
		$DSP->body .= $DSP->table_open(array('class' => 'tableBorder', 'border' => '0', 'style' => 'margin-top:18px; width:100%'));

		$DSP->body .= $DSP->tr()
			. $DSP->td('tableHeading', '', '2')
			. $LANG->line("open_statuses_title")
			. $DSP->td_c()
			. $DSP->tr_c();

		$DSP->body .= $DSP->tr()
			. $DSP->td('', '', '2')
			. "<div class='box' style='border-width:0 0 1px 0; margin:0; padding:10px 5px'><p>" . $LANG->line('open_statuses_info') . "</p></div>"
			. $DSP->td_c()
			. $DSP->tr_c();

		$DSP->body .= $DSP->tr()
			. '<td class="tableCellOne" style="width:60%; padding-top:8px; vertical-align:top;">'
			. $DSP->qdiv('defaultBold', $LANG->line("open_statuses_label"))
			. $DSP->td_c();
			
		$DSP->body .= $DSP->td('tableCellOne');
		
		foreach($statuses as $status_group)
	    {
	    	$DSP->body .= '<fieldset>'
	    	            . '<legend>'.$status_group['group_name'].'</legend>';
	    	
	    	foreach($status_group['statuses'] as $status_id => $status)
	    	{
	    		$checked = ( ! isset($current['open_statuses']))
	    			? ( $status == 'open' ? 'y' : 'n' )
	    			: ( in_array($status_id, $current['open_statuses']) ? 'y' : 'n' );
	    		
	    		$DSP->body .= '<label>'
	    			. $DSP->input_checkbox('open_statuses[]', $status_id, $checked)
	    			. " {$status}"
	    			. '</label>'
	    			. $DSP->br();
	    	}
	    	
	    	$DSP->body .= '</fieldset>'
	    	            . $DSP->br();
	    }
	    
	    $DSP->body .= $DSP->td_c()
		            . $DSP->tr_c();

		$DSP->body .= $DSP->table_c();
	    
	    
	    // Updates Setting

		$lgau_query = $DB->query("SELECT class
		                          FROM exp_extensions
		                          WHERE class = 'Lg_addon_updater_ext'
		                            AND enabled = 'y'
		                          LIMIT 1");
		$lgau_enabled = $lgau_query->num_rows ? TRUE : FALSE;
		$check_for_extension_updates = (( ! $lgau_enabled) OR ($current['check_for_extension_updates'] == 'y')) ? TRUE : FALSE;

		$DSP->body .= $DSP->table_open(
		                                   array(
		                                       'class'  => 'tableBorder',
		                                       'border' => '0',
		                                       'style' => 'margin-top:18px; width:100%'
		                                   )
		                               )

		            . $DSP->tr()
		            . $DSP->td('tableHeading', '', '2')
		            . $LANG->line("check_for_extension_updates_title")
		            . $DSP->td_c()
		            . $DSP->tr_c()

		            . $DSP->tr()
		            . $DSP->td('', '', '2')
		            . '<div class="box" style="border-width:0 0 1px 0; margin:0; padding:10px 5px"><p>'.$LANG->line('check_for_extension_updates_info').'</p></div>'
		            . $DSP->td_c()
		            . $DSP->tr_c()

		            . $DSP->tr()
		            . $DSP->td('tableCellOne', '60%')
		            . $DSP->qdiv('defaultBold', $LANG->line("check_for_extension_updates_label"))
		            . $DSP->td_c()

		            . $DSP->td('tableCellOne')
		            . '<select name="check_for_extension_updates"'.($lgau_enabled ? '' : ' disabled="disabled"').'>'
		            . $DSP->input_select_option('y', $LANG->line('yes'), ($check_for_extension_updates ? 'y' : ''))
		            . $DSP->input_select_option('n', $LANG->line('no'),  ($check_for_extension_updates ? '' : 'y'))
		            . $DSP->input_select_footer()
		            . ($lgau_enabled ? '' : NBS.NBS.NBS.$LANG->line('check_for_extension_updates_nolgau'))
		            . $DSP->td_c()
		            . $DSP->tr_c()

		            . $DSP->table_c();
	    
	    
	    // Close Form
	    
	    $DSP->body .= $DSP->qdiv('itemWrapperTop', $DSP->input_submit())
					. $DSP->form_c();
	}



	/**
	 * Save Settings
	 *
	 * @since version 1.2.0
	 */
	function save_settings()
	{
		global $DB, $PREFS;
		
		
		$settings = $this->get_all_settings();
		$current = $this->get_site_settings($settings);
		
		// Save new settings
		$settings[$PREFS->ini('site_id')] = $this->settings = array(
			'check_for_extension_updates' => isset($_POST['check_for_extension_updates']) ? $_POST['check_for_extension_updates'] : 'y',
			'open_statuses'               => $_POST['open_statuses'] ? $_POST['open_statuses'] : array()
		);
		
		$DB->query("UPDATE exp_extensions
		            SET settings = '".addslashes(serialize($settings))."'
		            WHERE class = '{$this->class_name}'");
		
		
		// Update Playa Data if open_statuses has changed
		
		if (( ! isset($current['open_statuses']))
			OR ($this->settings['open_statuses'] != $current['open_statuses']))
		{
			$this->update_all_playa_data();
		}
	}



	/**
	 * Update All Playa Data
	 *
	 * Updates all Playa Data based on new open_statuses setting.
	 *
	 * @since version 1.2.0
	 */
	function update_all_playa_data()
	{
		global $DB, $FNS;

		// Get all Playa fields
		$fields = $DB->query("SELECT field_id, group_id
			                  FROM exp_weblog_fields
			                  WHERE field_type = '{$this->type}'");
		
		foreach($fields->result as $field)
		{
			$field_name = 'field_id_'.$field['field_id'];
			
			// Get all Playa data
			$parents = $DB->query("SELECT entry_id, {$field_name} data
			                       FROM exp_weblog_data
			                       WHERE {$field_name} != ''");
			
			foreach($parents->result as $parent)
			{
				$rel_ids = $this->get_rel_ids($parent['data']);
				
				if ($rel_ids)
				{
					$children = $DB->query("SELECT r.rel_id, t.entry_id, t.title, t.status, w.status_group, s.status_id
					                        FROM exp_weblog_titles AS t,
					                             exp_weblogs AS w,
					                             exp_statuses AS s,
					                             exp_relationships AS r
					                        WHERE r.rel_id IN (".implode(',', $rel_ids).")
					                          AND t.entry_id = r.rel_child_id
					                          AND w.weblog_id = t.weblog_id
					                          AND ( w.status_group = 0 OR ( w.status_group = s.group_id AND s.status = t.status ))
					                        GROUP BY t.entry_id
					                        ORDER BY t.weblog_id, t.title");
					
					$playa_data = array();
					
					foreach($children->result as $child)
					{
						$key = array_search($child['rel_id'], $rel_ids);
						if ($key !== FALSE) {
							$status = $this->get_entry_status($child);
							$ref = $this->assemble_rel_reference($child['rel_id'], $child['title'], $status);
							$playa_data[$key] = $ref;
						}
					}
					
					$playa_data = array_filter($playa_data);
					ksort($playa_data);
					$this->update_playa_data($parent['entry_id'], $field_name, $playa_data);
				}
			}
		}
		
		// Clear relationships cache
		$FNS->clear_caching('relationships');
	}



	/**
	 * Activate Extension
	 *
	 * Resets all Playa exp_extensions rows
	 *
	 * @since version 1.0.0
	 */
	function activate_extension()
	{
		global $DB;
		
		
		// Get settings
		$settings = $this->get_all_settings();
		
		
		// Delete old hooks
		$DB->query("DELETE FROM exp_extensions
		            WHERE class = '{$this->class_name}'");
		
		
		// Add new extensions
		$ext_template = array(
			'class'    => $this->class_name,
			'settings' => $settings ? addslashes(serialize($settings)) : '',
			'priority' => 10,
			'version'  => $this->version,
			'enabled'  => 'y'
		);
		
		$extensions = array(
			// LG Addon Updater
			array('hook'=>'lg_addon_update_register_source',    'method'=>'register_my_addon_source'),
			array('hook'=>'lg_addon_update_register_addon',     'method'=>'register_my_addon_id'),

			// Admin > Field Groups
			array('hook'=>'show_full_control_panel_end',        'method'=>'edit_field_group'),
			array('hook'=>'publish_admin_edit_field_extra_row', 'method'=>'edit_custom_field'),
			array('hook'=>'sessions_start',                     'method'=>'save_custom_field'),

			// Publish / Edit
			array('hook'=>'publish_form_headers',               'method'=>'add_header_includes', 'priority'=>11),
			array('hook'=>'publish_form_field_unique',          'method'=>'assemble_field'),
			array('hook'=>'submit_new_entry_start',             'method'=>'modify_post_vars'),
			array('hook'=>'submit_new_entry_end',               'method'=>'update_relations'),

			// Delete
			array('hook'=>'delete_entries_start',               'method'=>'gather_mourning_parents'),
			array('hook'=>'delete_entries_loop',                'method'=>'relieve_mourning_parents'),

			// Modify Template
			array('hook'=>'weblog_entries_tagdata',             'method'=>'modify_template', 'priority'=>9)
		);
		
		foreach($extensions as $extension)
		{
			$ext = array_merge($ext_template, $extension);
			$DB->query($DB->insert_string('exp_extensions', $ext));
		}
	}



	/**
	 * Update Extension
	 *
	 * @param string   $current   Previous installed version of the extension
	 * @since version 1.0.0
	 */
	function update_extension($current='')
	{
		global $DB;

		
		if ($current == '' OR $current == $this->version)
		{
			return FALSE;
		}
		
		
		if ($current < '1.2.1')
		{
			$this->activate_extension();
		}
		else
		{
			$DB->query("UPDATE exp_extensions
			            SET version = '".$DB->escape_str($this->version)."'
			            WHERE class = '{$this->class_name}'");
		}
	}



	/**
	 * Disable Extension
	 *
	 * @since version 1.0.0
	 */
	function disable_extension()
	{
		global $DB;

		$DB->query("UPDATE exp_extensions
		            SET enabled='n'
		            WHERE class='{$this->class_name}'");
	}



	/**
	 * Register a New Addon Source
	 *
	 * @param  array   $sources   The existing sources
	 * @return array              The new source list
	 * @see    http://leevigraham.com/cms-customisation/expressionengine/lg-addon-updater/
	 * @author Leevi Graham <http://leevigraham.com>
	 * @since  version 1.1.0
	 */
	function register_my_addon_source($sources)
	{
		global $EXT;
		
	    // Check if we're not the only one using this hook
	    if($EXT->last_call !== FALSE)
	        $sources = $EXT->last_call;
	
	    // add a new source
	    // must be in the following format:
	    /*
	    <versions>
	        <addon id='LG Addon Updater' version='2.0.0' last_updated="1218852797" docs_url="http://leevigraham.com/" />
	    </versions>
	    */
	    if(( ! isset($this->settings['check_for_extension_updates'])) OR $this->settings['check_for_extension_updates'] == 'y')
	    {
	        $sources[] = 'http://brandon-kelly.com/downloads/versions.xml';
	    }
	    return $sources;
	
	}



	/**
	 * Register a New Addon
	 *
	 * @param  array   $addons   The existing sources
	 * @return array             The new addon list
	 * @see    http://leevigraham.com/cms-customisation/expressionengine/lg-addon-updater/
	 * @author Leevi Graham <http://leevigraham.com>
	 * @since  version 1.1.0
	 */
	function register_my_addon_id($addons)
	{
		global $EXT;
		
	    // Check if we're not the only one using this hook
	    if($EXT->last_call !== FALSE)
	        $addons = $EXT->last_call;
	
	    // add a new addon
	    // the key must match the id attribute in the source xml
	    // the value must be the addons current version
	    if(( ! isset($this->settings['check_for_extension_updates'])) OR $this->settings['check_for_extension_updates'] == 'y')
	    {
	        $addons[$this->class_name] = $this->version;
	    }
	    return $addons;
	}



	/**
	 * Get Gntry Status
	 *
	 * @param  array   $entry   Associative array of entry with keys 'status_group', 'status', and 'status_id'
	 * @return string           Entry's status based on open_statuses setting
	 * @since  version 1.2.0
	 */
	function get_entry_status($entry)
	{
		if (( ! $entry['status_group']) OR ($entry['status_group'] == '0')
			OR ( ! isset($this->settings['open_statuses'])))
		{
			$status = (strtolower($entry['status']) == 'open' ? 'open' : 'closed');
		}
		else
		{
			$status = in_array($entry['status_id'], $this->settings['open_statuses']) ? 'open' : 'closed';
		}
		
		return $status;
	}



	/**
	 * Assemble Relationship Reference
	 *
	 * @param  int|string   $rel_id   Relationship ID
	 * @param  string       $title    Entry title
	 * @param  string       $status   'open' or 'closed'
	 * @return string                 "[789] Title of Child Entry"
	 * @since  version 1.0.0
	 */
	function assemble_rel_reference($rel_id, $title, $status)
	{
		$whitelist = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z','0','1','2','3','4','5','6','7','8','9',' ');
		$simpleTitle = '';
		foreach(str_split($title) as $char) {
			if (in_array(strtolower($char), $whitelist)) $simpleTitle .= $char;
		}
		$closed = ($status AND $status != 'open') ? '!' : '';
		return "[{$closed}{$rel_id}] {$simpleTitle}";
	}



	/**
	 * Check For Playa Relationship
	 *
	 * @param  string       $data     Playa data
	 * @param  int|string   $rel_id   Relationship ID
	 * @return bool                   TRUE if $rel_id found in $data
	 * @since  version 1.0.0
	 */
	function check_for_playa_rel($data='', $rel_id)
	{
		return (preg_match("/\[(\!)?{$rel_id}\]/", $data));
	}



	/**
	 * Get Relationship IDs
	 *
	 * @param  string   $data            Playa data
	 * @param  bool     $ignore_closed   Only return "open" children if TRUE
	 * @return array                     All relationship IDs found in data (rel, mrel, or playa)
	 * @since  version 1.0.0
	 */
	function get_rel_ids($data='', $ignore_closed=FALSE)
	{
		$rel_ids = array();

		$lines = array_filter(preg_split("/[\r\n]+/", $data));
		if (count($lines))
		{
			foreach($lines as $line)
			{
				if (preg_match('/\[(\!)?(\d+)\]/', $line, $matches))
				{
					if ( ! $ignore_closed OR ! $matches[1])
					{
						$rel_ids[] = $matches[2];
					}
				}
				else
				{
					$rel_ids[] = $line;
				}
			}
		}

		return $rel_ids;
	}



	/**
	 * Get Playa Field Names
	 *
	 * @return array   All field_id_XXX's with the Playa field type
	 * @since  version 1.0.0
	 */
	function get_playa_fields()
	{
		global $PLAYA_FIELDS, $DB;

		if (isset($PLAYA_FIELDS))
		{
			return $PLAYA_FIELDS;
		}


		$PLAYA_FIELDS = array();

		$query = $DB->query("SELECT field_id
			                 FROM exp_weblog_fields
			                 WHERE field_type = '{$this->type}'");

		if ($query->num_rows)
		{
			foreach($query->result as $playa_field)
			{
				$PLAYA_FIELDS[] = 'field_id_'.$playa_field['field_id'];
			}
		}

		return $PLAYA_FIELDS;
	}



	/**
	 * Update Entry's Playa Data
	 *
	 * @param int|string   $entry_id     Parent entry ID
	 * @param string       $field_name   'field_id_XXX'
	 * @param array        $playa_data   Child relationship references
	 * @since version 1.0.0
	 */
	function update_playa_data($entry_id, $field_name, $playa_data)
	{
		global $DB;

		for ($i=0; $i<count($playa_data); $i++)
		{
			$playa_data[$i] = addslashes($playa_data[$i]);
		}

		$DB->query("UPDATE exp_weblog_data
		            SET {$field_name} = '".implode('\r', $playa_data)."'
		            WHERE entry_id = {$entry_id}");
	}



	/**
	 * Get Entry's Parents
	 *
	 * @param  int|string   $child_id   Child entry ID
	 * @return array                    Array of associative arrays holding 'rel_id', 'parent_id', 'field_name', and 'field_data' for each parent
	 * @since  version 1.0.0
	 */
	function get_parents($child_id)
	{
		global $DB;

		$parents = array();


		// Get all rels (rel, mrel, or playa)
		$rels = $DB->query("SELECT rel_id, rel_parent_id
		                    FROM exp_relationships
		                    WHERE rel_child_id = '{$child_id}'");

		if ($rels->num_rows)
		{
			// Filter out non-Playa rels

			$playa_fields = $this->get_playa_fields();

			if (count($playa_fields))
			{
				foreach($rels->result as $rel)
				{
					$rel_id = $rel['rel_id'];
					$rel_parent_id = $rel['rel_parent_id'];

					$query = $DB->query("SELECT ".implode(', ', $playa_fields)."
					                     FROM exp_weblog_data
					                     WHERE entry_id = {$rel_parent_id}
					                       AND ".implode(" LIKE '%[{$rel_id}]%' OR ", $playa_fields)." LIKE '%[{$rel_id}]%' OR ".
					                             implode(" LIKE '%[!{$rel_id}]%' OR ", $playa_fields)." LIKE '%[!{$rel_id}]%'
					                     LIMIT 1");
					if ($query->num_rows)
					{
						foreach($query->row as $field_name => $field_data)
						{
							$rel_ids = $this->get_rel_ids($field_data);
							if (in_array($rel_id, $rel_ids))
							{
								$parents[] = array(
									'rel_id'     => $rel_id,
									'parent_id'  => $rel_parent_id,
									'field_name' => $field_name,
									'field_data' => $field_data
								);

								break;
							}
						}
					}
				}
			}
		}

		return $parents;
	}



	/**
	 * Edit Field Group
	 *
	 * @param  string   $out   The content of the admin page to be outputted
	 * @return string          Modified $out
	 * @see    http://expressionengine.com/developers/extension_hooks/show_full_control_panel_end/
	 * @author Mark Huot <docs@markhuot.com>
	 * @since  version 1.0.0
	 */
	function edit_field_group($out)
	{
		global $DB, $EXT, $LANG;

		// Check if we're not the only one using this hook
		if($EXT->last_call !== false)
		{
			$out = $EXT->last_call;
		}

		if(preg_match_all("/C=admin&amp;M=blog_admin&amp;P=edit_field&amp;field_id=(\d*).*?<\/td>.*?<td.*?>.*?<\/td>.*?<\/td>/is", $out, $matches))
		{
			foreach($matches[1] as $key=>$field_id)
			{
				$query = $DB->query("SELECT field_type
				                     FROM exp_weblog_fields
				                     WHERE field_id='{$field_id}'
				                     LIMIT 1");

				if($query->row['field_type'] == $this->type)
				{
					$replace = preg_replace("/(<td.*?<td.*?>).*?<\/td>/si", "$1{$this->name}</td>", $matches[0][$key]);
					$out = str_replace($matches[0][$key], $replace, $out);
				}
			}
			return $out;
		}
		else
		{
		    return $out;
		}
	}



	/**
	 * Edit Custom Field
	 *
	 * @param  array    $data   The data about this field from the database
	 * @param  string   $r      The current contents of the page
	 * @return string           Modified $r
	 * @see    http://expressionengine.com/developers/extension_hooks/publish_admin_edit_field_extra_row/
	 * @author Mark Huot <docs@markhuot.com>
	 * @since  version 1.0.0
	 */
	function edit_custom_field($data, $r)
	{
		global $EXT, $LANG, $DB, $PREFS, $DSP;

		// Check if we're not the only one using this hook
		if($EXT->last_call !== false)
		{
			$r = $EXT->last_call;
		}
		
		// Load language files
		$LANG->fetch_language_file('playa');

		// Is MSM enabled?
		$msm = ($PREFS->ini('multiple_sites_enabled') == 'y') ? TRUE : FALSE;

		// Get the current Site ID
		$site_id = $PREFS->ini('site_id');

		// Add playa <option /> to Field Type drop-down

		$selected = (isset($data['field_type']) AND $data['field_type'] == $this->type) ? " selected=\"true\"" : "";
		$r = preg_replace("/(<select.*?name=.field_type.*?value=.select.*?[\r\n])/is", "$1<option value=\"{$this->type}\"{$selected}>{$this->name}</option>\r", $r);


		// --------------------------------
		//  Add custom JS
		// --------------------------------

		// Tell other Field Type handlers to hide playa blocks on select
		//     - Unfortunately, this won't get added to any other custom field types
		//       that get added *after* this one :-(

		$r = preg_replace("/(id\s*==.*?)\}/s", "$1
			document.getElementById('playa_block1').style.display = \"none\";
			document.getElementById('playa_block2').style.display = \"none\";
		}", $r);

		// Add handler for playa Field Type

		$r = preg_replace("/(id\s*==\s*.rel.*?})/is", "$1
		else if (id == '{$this->type}')
		{
			document.getElementById('playa_block1').style.display = \"block\";
			document.getElementById('playa_block2').style.display = \"block\";
			document.getElementById('rel_block').style.display = \"none\";
			document.getElementById('select_block').style.display = \"none\";
			document.getElementById('pre_populate').style.display = \"none\";
			document.getElementById('text_block').style.display = \"none\";
			document.getElementById('textarea_block').style.display = \"none\";
			document.getElementById('date_block').style.display = \"none\";
			document.getElementById('rel_block').style.display = \"none\";	
			document.getElementById('relationship_type').style.display = \"none\";	
			document.getElementById('formatting_block').style.display = \"none\";
			document.getElementById('formatting_unavailable').style.display = \"block\";
		}", $r);


		// --------------------------------
		//  Set proper built-in block displays if field_type == playa
		// --------------------------------

		if (isset($data['field_type']) AND $data['field_type'] == $this->type)
		{
			preg_match("/(id=.formatting_block.*?display:\s*)block/", $r, $formatting_unavailable);
			if(count($formatting_unavailable) > 0)
			{
				$r = str_replace($formatting_unavailable[0], $formatting_unavailable[1] .= "none", $r);
			}
			preg_match("/(id=.formatting_unavailable.*?display:\s*)none/", $r, $formatting_unavailable);
			if(count($formatting_unavailable) > 0)
			{
				$r = str_replace($formatting_unavailable[0], $formatting_unavailable[1] .= "block", $r);
			}
		}


		// --------------------------------
		//  Add custom blocks
		// --------------------------------

		if (isset($data['field_type']) AND $data['field_type'] == $this->type)
		{
			$display = 'block';
			$field_playa_max = (isset($data['field_maxl'])) ? $data['field_maxl'] : '';
			$field_playa_size = (isset($data['field_ta_rows'])) ? $data['field_ta_rows'] : '10';
			$field_playa_blogs = (isset($data['field_list_items']))
				? array_filter(explode(',', $data['field_list_items']))
				: array();
		}
		else
		{
			$display = 'none';
			$field_playa_max = '';
			$field_playa_size = '10';
			$field_playa_blogs = array();
		}


		$block1 = '<div id="playa_block1" style="display:'.$display.';">'
		        . $DSP->div('itemWrapper')
		        . NBS.NBS.NBS.'<label>'
		        . $DSP->input_text('field_playa_maxl', $field_playa_max, '4', '3', 'input', '30px')
			    . NBS.$LANG->line('field_playa_max_label')
			    . '</label>'
			    . $DSP->div_c()
			    . $DSP->div('itemWrapper')
			    . NBS.NBS.NBS.'<label>'
			    . $DSP->input_text('field_playa_size', $field_playa_size, '4', '3', 'input', '30px')
			    . NBS.$LANG->line('field_playa_size_label')
			    . '</label>'
			    . $DSP->div_c()
			    . $DSP->div_c();

		// Add $block1 above div#textarea_block
		$r = preg_replace("/(<div\s*id=.textarea_block.)/", "$block1$1", $r);


		$blogs = $DB->query('SELECT w.weblog_id, s.site_label, w.blog_title
		                     FROM exp_weblogs AS w,
		                          exp_sites   AS s
		                     WHERE s.site_id = w.site_id '
		                  . ($msm ? '' : "AND s.site_id = '{$site_id}' ")
		                  . 'ORDER BY s.site_label, w.blog_title ASC');

		$block2 = '<div id="playa_block2" style="display:'.$display.';">'
		        . '<div id="playa_block2_blog">'
			    . $DSP->div('itemWrapper')
			    . $DSP->qdiv('defaultBold', $LANG->line('field_playa_blogs_label'))
			    . $DSP->div_c();
		
		if ($blogs->num_rows > 0)
		{
			$block2 .= $DSP->input_select_header('field_playa_blogs[]', 'y', ($blogs->num_rows < 15 ? $blogs->num_rows : 15));
			
			foreach($blogs->result as $blog)
			{
				// Prefix the blog name with the site name if MSM is enabled
				$blog_name = ($msm ? $blog['site_label'].' - ' : '') . $blog['blog_title'];
				
				$selected = in_array($blog['weblog_id'], $field_playa_blogs) ? 1 : 0;
				$block2 .= $DSP->input_select_option($blog['weblog_id'], $blog_name, $selected);
			}

			$block2 .= $DSP->input_select_footer();
		}
		else
		{
			$block2 .= $DSP->qdiv('highlight', $LANG->line('no_weblogs_exist'));
		}
		
		
		// Sorting
		
		function input_select($name, $value, $options)
		{
			global $DSP, $LANG;
			
			$r = $DSP->input_select_header($name);
			foreach($options as $option_value => $option_name)
			{
				if ( ! is_string($option_value)) $option_value = $option_name;
				$r .= $DSP->input_select_option($option_value, $option_name, ($value == $option_value ? 1 : 0));
			}
			$r .= $DSP->input_select_footer();
			
			return $r;
		}
		
		$block2 .= $DSP->div_c()
		
		         . '<div id="playa_block_sort">'
		         . $DSP->div('itemWrapper')
		         . $DSP->div('itemWrapper')
		         . $DSP->qdiv('defaultBold', $LANG->line('display_criteria'))
		         . $DSP->div_c()
		         
		         . $LANG->line('sort_by').NBS
		         . input_select('field_playa_orderby', $data['field_related_orderby'],
		                            array(
		                                'title'      => $LANG->line('title'),
		                                'entry_date' => $LANG->line('date'),
		                                'edit_date'  => $LANG->line('edit_date')
		                            ))
		         
		         . NBS.$LANG->line('in').NBS
		         . input_select('field_playa_sort', $data['field_related_sort'],
		                            array(
		                                'asc'  => $LANG->line('sort_asc'),
		                                'desc' => $LANG->line('sort_desc')
		                            ))
		         
		         . NBS.$LANG->line('limit').NBS
		         . input_select('field_playa_max', $data['field_related_max'],
		                            array('0'=>'All', '25', '50', '100', '250', '500', '1000'))
		         
		         . $DSP->div_c()
		         . $DSP->div_c()
		         . $DSP->div_c();


		// Add $block2 above div#date_block
		$r = preg_replace("/(<div\s*id=.date_block.)/", "$block2$1", $r);


		return $r;
	}



	/**
	 * Save Custom Field
	 *
	 * @see   http://expressionengine.com/developers/extension_hooks/sessions_start/
	 * @since version 1.0.0
	 */
	function save_custom_field()
	{
		$is_playa = (isset($_POST['field_type']) AND $_POST['field_type'] == $this->type) ? TRUE : FALSE;
		
		if ($is_playa)
		{
			$_POST['field_pre_populate'] = 'n';
			$_POST['field_pre_blog_id'] = '0';
			$_POST['field_pre_field_id'] = '0';
			$_POST['field_fmt'] = 'xhtml';
			$_POST['field_show_fmt'] = 'n';
			
			$_POST['field_list_items'] = isset($_POST['field_playa_blogs'])
				? implode(',', $_POST['field_playa_blogs'])
				: '';
		}
		if (isset($_POST['field_playa_blogs']))
		{
			unset($_POST['field_playa_blogs']);
		}
		$i = 0;
		while(isset($_POST["field_playa_blogs_{$i}"]))
		{
			unset($_POST["field_playa_blogs_{$i}"]);
			$i ++;
		}
		
		$fields = array(
			array('playa_post'=>'field_playa_maxl',    'col'=>'field_maxl',            'default'=>'0'),
			array('playa_post'=>'field_playa_size',    'col'=>'field_ta_rows',         'default'=>'10'),
			array('playa_post'=>'field_playa_orderby', 'col'=>'field_related_orderby', 'default'=>'title'),
			array('playa_post'=>'field_playa_sort',    'col'=>'field_related_sort',    'default'=>'asc'),
			array('playa_post'=>'field_playa_max',     'col'=>'field_related_max',     'default'=>'0')
		);
		
		foreach($fields as $field)
		{
			if ($is_playa)
			{
				$_POST[$field['col']] = isset($_POST[$field['playa_post']])
					? $_POST[$field['playa_post']]
					: $field['default'];
			}
			if (isset($_POST[$field['playa_post']]))
			{
				unset($_POST[$field['playa_post']]);
			}
		}
	}



	/**
	 * Add Header Includes
	 *
	 * @return string   Modified header
	 * @see    http://expressionengine.com/developers/extension_hooks/publish_form_headers/
	 * @since  version 1.0.0
	 */
	function add_header_includes()
	{
		global $EXT, $DB, $PREFS, $DSP;

		// Check if we're not the only one using this hook
		$r = ($EXT->last_call !== FALSE) ? $EXT->last_call : '';

		$playa_dir = './extensions/playa/';

		$r .= "\n\n<link rel=\"stylesheet\" type=\"text/css\" href=\"{$playa_dir}playa.css?v={$this->version}\" />\n" .
		      "<script type=\"text/javascript\" src=\"{$playa_dir}playa.js?v='.$this->version.'\"></script>\n" .
		      "<script type=\"text/javascript\">var playa_dir = \"{$playa_dir}\";</script>\n" .
		      "<!--[if IE]><script type=\"text/javascript\">playa_ie=7</script><![endif]-->\n" .
		      "<!--[if lte IE 6]><script type=\"text/javascript\">playa_ie=6</script><![endif]-->\n\n";
		
		$query = $DB->query('SELECT status_id, highlight
		                     FROM exp_statuses');

		if ($query->num_rows)
		{
			$r .= "<style type=\"text/css\">\n";
			foreach($query->result as $status)
			{
				$highlight = ($status['highlight']) ? $status['highlight'] : '000000';
				$r .= "\t.playa-statusid-{$status['status_id']} { border-left-color:#{$highlight}; }\n";
			}
			$r .= "</style>\n\n";
		}

		return $r;
    }



	/**
	 * Assemble Field
	 *
	 * @param  array    $row          Parameters for the field from the database
	 * @param  string   $field_data   If entry is not new, this will have field's current value
	 * @return string                 The Playa field
	 * @see    http://expressionengine.com/developers/extension_hooks/publish_form_field_unique/
	 * @since  version 1.0.0
	 */
	function assemble_field($row, $field_data)
	{
		global $DB, $DSP, $EXT, $LANG;
		
		if ($row['field_type'] != $this->type)
		{
			return ($EXT->last_call !== false) ? $EXT->last_call : '';
		}


		// --------------------------------
		//  Get options data
		// --------------------------------

		$no_related_entries = '<table border="0" cellpadding="0" cellspacing="0" style="margin-bottom: 0; width: 100%;"><tr><td>'
		                    . $DSP->qdiv('highlight_alt', $LANG->line('no_related_entries'))
		                    . '</tr></td></table>';

		$playa_id = 'playa-'.$row['field_id'];
		$field_id = 'field_id_'.$row['field_id'];

		$rel_blog_ids = array_filter(explode(',', $row['field_list_items']));
		if ( ! count($rel_blog_ids)) return $no_related_entries;

		$blog_options = $DB->query("SELECT weblog_id, blog_title
		                            FROM exp_weblogs
		                            ORDER BY blog_title");
		if ( ! $blog_options->num_rows) return $no_related_entries;
		
		$orderby = in_array($row['field_related_orderby'], array('title', 'entry_date', 'edit_date'))
			? $row['field_related_orderby']
			: 'title';
		
		$limit = in_array($row['field_related_max'], array('25', '50', '100', '250', '500', '1000'))
			? "LIMIT {$row['field_related_max']}"
			: '';
		
		$sort = in_array($row['field_related_sort'], array('asc', 'desc'))
			? strtoupper($row['field_related_sort'])
			: 'ASC';
		
		$entry_options = $DB->query("SELECT t.entry_id, t.weblog_id, t.title, t.status, w.status_group, s.status_id
		                             FROM exp_weblog_titles AS t,
		                                  exp_weblogs       AS w,
		                                  exp_statuses      AS s
		                             WHERE t.weblog_id IN (".implode(',', $rel_blog_ids).")
		                               AND t.weblog_id = w.weblog_id
		                               AND ( w.status_group = 0 OR ( w.status_group = s.group_id AND s.status = t.status ))
		                             GROUP BY t.entry_id
		                             ORDER BY t.weblog_id ASC, t.{$orderby} {$sort}
		                             {$limit}");
		
		if ( ! $entry_options->num_rows) return $no_related_entries;


		// --------------------------------
		//  Get selected entry ids
		// --------------------------------

		$selected_entry_ids = array();

		if ( ! isset($_POST['field_id_'.$row['field_id']]) OR (isset($which) AND $which == 'save'))
		{
			$rel_ids = $this->get_rel_ids($field_data);
			if (count($rel_ids))
			{
				$rels = $DB->query("SELECT rel_id, rel_child_id
				                    FROM exp_relationships
				                    WHERE rel_id IN (".implode(',', $rel_ids).")");

				if ($rels->num_rows)
				{
					$selected_entry_ids_by_rel_id = array();

					foreach($rels->result as $rel)
					{
						$selected_entry_ids_by_rel_id[$rel['rel_id']] = $rel['rel_child_id'];
					}

					foreach($rel_ids as $rel_id)
					{
						if (array_key_exists($rel_id, $selected_entry_ids_by_rel_id))
						{
							$selected_entry_ids[] = $selected_entry_ids_by_rel_id[$rel_id];
						}
					}
				}
			}
		}
		else
		{
			$selected_entry_ids = explode(',', $_POST['field_id_'.$row['field_id']]);
		}


		// --------------------------------
		//  Assemble $options and $selections arrays
		// --------------------------------

		$options = array();
		$selections = array();

		foreach($blog_options->result as $blog)
		{
			$options[$blog['weblog_id']] = array(
				'title' => $blog['blog_title'],
				'entries' => array()
			);
		}

		foreach($entry_options->result as $entry)
		{
			if ( ! isset($options[$entry['weblog_id']])) continue;
			
			$status = $this->get_entry_status($entry);
			if (( $entry['status_group']) AND ($entry['status_group'] != '0'))
			{
				$status .= ' playa-statusid-' . $entry['status_id'];
			}

			$options[$entry['weblog_id']]['entries'][$entry['entry_id']] = array(
				'title'  => $entry['title'],
				'status' => $status
			);

			if ( ! in_array($entry['entry_id'], $selected_entry_ids))
			{
				$options[$entry['weblog_id']]['entries'][$entry['entry_id']]['selected'] = FALSE;
			}
			else
			{
				$selections[$entry['entry_id']] = $options[$entry['weblog_id']]['entries'][$entry['entry_id']];
				$options[$entry['weblog_id']]['entries'][$entry['entry_id']]['selected'] = TRUE;
			}
		}


		// --------------------------------
		//  Assemble ul.options.list
		// --------------------------------

		$options_list = '';

		foreach($options as $blog_id => $blog)
		{
			if ( ! count($blog['entries'])) continue;

			$options_list .= '<li class="heading"><h4>'.$blog['title'].'</h4><ul>';

			foreach($blog['entries'] as $entry_id => $entry)
			{
				$selected = ($entry['selected']) ? ' selected' : '';

				$options_list .= '<li id="'.$playa_id.'-option-'.$entry_id.'" '.
				                     'class="playa-status-'.$entry['status'].$selected.'" '.
				                     'title="'.$entry['title'].'">'.
				                         $entry['title'].
				                     '</li>';
			}

			$options_list .= '</ul></li>';
		}

		if ($options_list == '') return $no_related_entries;


		// --------------------------------
		//  Assemble ul.selections.list
		// --------------------------------

		$selections_list = '';

		foreach($selected_entry_ids as $entry_id)
		{
			if ( ! isset($selections[$entry_id])) continue;

			$selections_list .= "<li id=\"{$playa_id}-option-{$entry_id}-selected\" ".
			                        "class=\"playa-status-{$selections[$entry_id]['status']}\" ".
			                        "title=\"{$selections[$entry_id]['title']}\">".
			                            $selections[$entry_id]['title'].'<div class="dragHandle"></div></li>';
		}


		// --------------------------------
		//  Assemble and return HTML
		// --------------------------------

		$ul_height = (($row['field_ta_rows']) ? (int) $row['field_ta_rows'] : 10) * 21 - 1;

		$r = '<table border="0" cellpadding="0" cellspacing="0" class="playa" id="'.$playa_id.'">
		          <tr>
		              <td class="options list">
		                  <ul style="height: '.$ul_height.'px;">'.$options_list.'</ul>
		              </td>
		              <td class="btns">
		                  <input type="button" class="select-options" value="Add -&gt" />
		                  <input type="button" class="deselect-options" value="&lt;- Remove" />
		              </td>
		              <td class="selections list">
		                  <ul style="height: '.$ul_height.'px;">'.$selections_list.'</ul>
		              </td>
		          </tr>
		          <tr>
		              <td class="options info"></td>
		              <td></td>
		              <td class="selections info"></td>
		          </tr>
		      </table>
		      <div id="'.$playa_id.'-console" style="font-family: monospace;"></div>
		      <input type="hidden" id="'.$field_id.'" name="'.$field_id.'" value="'.implode(',', $selected_entry_ids).'" />
		      <input type="hidden" name="field_ft_'.$row['field_id'].'" value="'.$row['field_fmt'].'" />
		      <script type="text/javascript">playa_init("'.$row['field_id'].'", '.$row['field_maxl'].');</script>';


		return $r;

	}



	/**
	 * Modify Post Vars
	 *
	 * Store Playa-specific $_POST vars in global $SUBMITTED_PLAYA for later use, and then unset them.
	 *
	 * @see   http://expressionengine.com/developers/extension_hooks/submit_new_entry_start/
	 * @since version 1.0.0
	 */
	function modify_post_vars()
	{
		global $DB, $SUBMITTED_PLAYA;

		if ( ! isset($SUBMITTED_PLAYA)) $SUBMITTED_PLAYA = array();


		// Get field ids for each Playa field

		$query = $DB->query("SELECT field_id
		                     FROM exp_weblog_fields
		                     WHERE field_type = '{$this->type}'");


		// Save Playa $_POST data in $SUBMITTED_PLAYA and unset it from $_POST

		foreach($query->result as $field)
		{
			$field_id = 'field_id_'.$field['field_id'];
			if (isset($_POST[$field_id]))
			{
				$SUBMITTED_PLAYA[$field_id] = array_filter(explode(',', $_POST[$field_id]));
				unset($_POST[$field_id]);
			}
			else
			{
				$SUBMITTED_PLAYA[$field_id] = array();
			}
		}
	}



	/**
	 * Update Relationships
	 *
	 * After the entry has been saved (and we have its entry ID), update its Playa relationships using the previously-set $SUBMITTED_PLAYA global variable.
	 *
	 * @param int|string   $entry_id   Entry's ID
	 * @param array        $data       Array of data about entry (title, url_title)
	 * @see   http://expressionengine.com/developers/extension_hooks/submit_new_entry_end/
	 * @since version 1.0.0
	 */
	function update_relations($entry_id, $data)
	{
		global $DB, $FNS, $SUBMITTED_PLAYA;
		
		
		$clear_cache = FALSE;

		
		if (isset($SUBMITTED_PLAYA) AND count($SUBMITTED_PLAYA))
		{
			// Get all child titles

			$children = array();
	
			$all_child_ids = array();
			foreach($SUBMITTED_PLAYA as $field_id => $child_ids)
			{
				if (count($child_ids))
				{
					foreach($child_ids as $child_id)
					{
						if ( ! in_array($child_id, $all_child_ids))
						{
							$all_child_ids[] = $child_id;
						}
					}
				}
			}
			if (count($all_child_ids))
			{
				$query = $DB->query("SELECT t.entry_id, t.title, t.status, w.status_group, s.status_id
				                     FROM exp_weblog_titles AS t, exp_weblogs AS w, exp_statuses AS s
				                     WHERE entry_id IN (".implode(',', $all_child_ids).")
				                       AND t.weblog_id = w.weblog_id
				                       AND ( w.status_group = 0 OR ( w.status_group = s.group_id AND s.status = t.status ))
				                     GROUP BY t.entry_id");
	
				if ($query->num_rows)
				{
					foreach($query->result as $child)
					{
						$children[$child['entry_id']] = array(
							'title'  => $child['title'],
							'status' => $this->get_entry_status($child)
						);
					}
				}
			}


			foreach($SUBMITTED_PLAYA as $field_id => $child_ids)
			{
				// Get the field type ("blog")
				$field_type_q = $DB->query("SELECT field_related_to type
				                            FROM exp_weblog_fields
				                            WHERE field_id = ".substr($field_id, 9));
				$field_type = $field_type_q->num_rows ? $field_type_q->row['type'] : 'blog';
				
				
				// Define existing_rels arrays
				//     saved as [rel_child_id => rel_id]
				$existing_rels_to_stay = array(); 
				$existing_rels_to_delete = array();
				
				// Get existing rel_id's
				
				$existing_cell_data = $DB->query("SELECT {$field_id} field_data
				                                  FROM exp_weblog_data
				                                  WHERE entry_id = {$entry_id}
				                                    AND {$field_id} != ''
				                                  LIMIT 1");
	
				if ($existing_cell_data->num_rows)
				{
					$existing_rel_ids = $this->get_rel_ids($existing_cell_data->row['field_data']);
					
					if (count($existing_rel_ids))
					{
						// Get their rel_child_id's
						
						$existing_rels = $DB->query("SELECT rel_id, rel_child_id
						                             FROM exp_relationships
						                             WHERE rel_id IN (".implode(',', $existing_rel_ids).")");
						
						if ($existing_rels->num_rows)
						{
							foreach($existing_rels->result as $existing_rel)
							{
								if (in_array($existing_rel['rel_child_id'], $child_ids))
								{
									$existing_rels_to_stay[$existing_rel['rel_child_id']] = $existing_rel['rel_id'];
								}
								else
								{
									$existing_rels_to_delete[$existing_rel['rel_child_id']] = $existing_rel['rel_id'];
								}
							}
						}
						
					}
				}
				
				
				// Delete relationships marked for deletion:
				if (count($existing_rels_to_delete))
				{
					$DB->query("DELETE FROM exp_relationships
					            WHERE rel_id IN (".implode(',',$existing_rels_to_delete).")");
					
					if ( ! $clear_cache) $clear_cache = TRUE;
				}
	
	
	
				// Assemble Playa Data
				
				$playa_data = array();
	
				foreach($child_ids as $child_id)
				{
					if (array_key_exists($child_id, $existing_rels_to_stay))
					{
						// Just grab the rel_id
						$rel_id = $existing_rels_to_stay[$child_id];
					}
					else
					{
						// Compile the new relationship
						$reldata = array(
							'type'       => $field_type,
							'parent_id'  => $entry_id,
							'child_id'   => $child_id,
							'related_id' => $data['weblog_id']
						);
						$rel_id = $FNS->compile_relationship($reldata, TRUE);
						
						if ( ! $clear_cache) $clear_cache = TRUE;
					}
					
					// Add the rel_id to $playa_data
					$playa_data[] = $this->assemble_rel_reference($rel_id, $children[$child_id]['title'], $children[$child_id]['status']);
				}
	
				$this->update_playa_data($entry_id, $field_id, $playa_data);
			}
			
		} // end if ! $SUBMITTED_PLAYA


		// Update parents unless it's a new entry
		if ( ! array_key_exists('entry_id', $data))
		{
			// get parents
			$parents = $this->get_parents($entry_id);
			if (count($parents))
			{
				// get Playa status
				$query = $DB->query("SELECT t.status, w.status_group, s.status_id
				                     FROM exp_weblog_titles AS t, exp_weblogs AS w, exp_statuses AS s
				                     WHERE entry_id = {$entry_id}
				                       AND t.weblog_id = w.weblog_id
				                       AND ( w.status_group = 0 OR ( w.status_group = s.group_id AND s.status = t.status ))
				                     GROUP BY t.entry_id
				                     LIMIT 1");
				$status = $this->get_entry_status($query->row);
				
				
				foreach($parents as $parent)
				{
					$lines = array_filter(preg_split("/[\r\n]+/", $parent['field_data']));
					$new_lines = array();
					foreach($lines as $line)
					{
						$new_lines[] = ($this->check_for_playa_rel($line, $parent['rel_id']))
							? $this->assemble_rel_reference($parent['rel_id'], $data['title'], $status)
							: $line;
					}
					$this->update_playa_data($parent['parent_id'], $parent['field_name'], $new_lines);
				}
				
				if ( ! $clear_cache) $clear_cache = TRUE;
			}
		}
		
		
		if ($clear_cache)
		{
			// Clear Relationships cache
			$FNS->clear_caching('relationships');
		}
	}



	/**
	 * Gather Mourning Parents
	 *
	 * Gather all Playa relationships whose children are about to be deleted, and save them in the global variable $PLAYA_MOURNING_PARENTS for later use
	 *
	 * @see   http://expressionengine.com/developers/extension_hooks/delete_entries_start/
	 * @since version 1.0.0
	 */
	function gather_mourning_parents()
	{
		global $DB, $PLAYA_MOURNING_PARENTS;

		$playa_fields = $this->get_playa_fields();
		$PLAYA_MOURNING_PARENTS = array();


		if (count($playa_fields))
		{
			foreach ($_POST as $key => $val)
			{
				if (strstr($key, 'delete') AND ! is_array($val) AND is_numeric($val))
				{                    
					$parents = $this->get_parents($val);

					if (count($parents))
					{
						$PLAYA_MOURNING_PARENTS[$val] = $parents;
					}
				}
			}
		}
	}



	/**
	 * Relieve Mourning Parents
	 *
	 * After an entry and its relationships have been deleted, delete any references to the now non-existent Playa relationships
	 *
	 * @param int|string   $val         Entry ID for entry being deleted during this loop
	 * @param int|string   $weblog_id   Weblog ID for entry being deleted
	 * @see   http://expressionengine.com/developers/extension_hooks/delete_entries_loop/
	 * @since version 1.0.0
	 */
	function relieve_mourning_parents($val, $weblog_id)
	{
		global $DB, $PLAYA_MOURNING_PARENTS;

		$playa_fields = $this->get_playa_fields();

		if (count($playa_fields) AND isset($PLAYA_MOURNING_PARENTS[$val]))
		{
			foreach($PLAYA_MOURNING_PARENTS[$val] as $parent)
			{
				$lines = array_filter(preg_split("/[\r\n]+/", $parent['field_data']));
				$new_lines = array();
				foreach($lines as $line)
				{
					if ( ! $this->check_for_playa_rel($line, $parent['rel_id']))
					{
						$new_lines[] = $line;
					}
				}
				
				$DB->query("UPDATE exp_weblog_data
			                SET {$parent['field_name']} = '".implode('\r', $new_lines)."'
			                WHERE entry_id = '{$parent['parent_id']}'");
			}
		}
	}



	/**
	 * Modify Template
	 *
	 * Replace meaningless Playa-based $related_entries with real Playa children.
	 *
	 * Huge props & thanks to Mark Huot for figuring all this out.
	 *
	 * @param  string   $tagdata   The Weblog Entries tag data
	 * @param  array    $row       Array of data for the current entry
	 * @param  object   $weblog    The current Weblog object including all data relating to categories and custom fields
	 * @return string              Modified $tagdata
	 * @see    http://expressionengine.com/developers/extension_hooks/weblog_entries_tagdata/
	 * @author Mark Huot <docs@markhuot.com>
	 * @author Brandon Kelly <me@brandon-kelly.com>
	 * @since  version 1.0.0
	 */
	function modify_template($tagdata, $row, &$weblog)
	{
		global $DB, $FNS, $PREFS, $TMPL, $EXT, $Weblog, $PLAYA_LIMITS;

		if($EXT->last_call !== false)
		{
			$tagdata = $EXT->last_call;
		}

		if ( ! isset($PLAYA_LIMITS)) $PLAYA_LIMITS = array();

		$site_id = $PREFS->ini('site_id');
		$count = 0;

		$reference = isset($Weblog->query) ? "Weblog" : "weblog";

		foreach($TMPL->related_data as $key => $rel_data)
		{
			// weblog->cfields holds field name/id pairs

			if( ! isset($$reference->cfields[$site_id])) continue;
			$field_id = $$reference->cfields[$site_id][$rel_data['field_name']];

			preg_match('/'.LD.'related_entries:attributes(.*?)'.RD.'/', $rel_data['tagdata'], $attributes);
			$attributes = (isset($attributes[1]) AND count($attributes[1]) > 0)
			              ? $FNS->assign_parameters($attributes[1])
			              : array();

			$TMPL->related_data[$key]['tagdata'] = preg_replace('/'.LD.'related_entries:attributes(.*?)'.RD.'/', '', $TMPL->related_data[$key]['tagdata']);

			$inner_build = '';
			$field_data = $this->get_rel_ids($row['field_id_'.$field_id], TRUE);

			if (count($field_data))
			{
				if ( ! isset($PLAYA_LIMITS[$field_id]))
				{
					$query = $DB->query("SELECT field_maxl
					                     FROM exp_weblog_fields
					                     WHERE field_id='{$field_id}'
					                       AND field_type='{$this->type}'");

					$PLAYA_LIMITS[$field_id] = ($query->num_rows) ? $query->row['field_maxl'] : 0;
				}

				foreach($field_data as $count => $rel_entry)
				{
					if (isset($attributes['limit']) AND $count >= $attributes['limit']) break;
					if ($PLAYA_LIMITS[$field_id] AND $count >= $PLAYA_LIMITS[$field_id]) break;

					$$reference->rfields[$rel_data['field_name']] = $field_id;
					$$reference->related_entries[] = $rel_entry.'_'.$rel_data['marker'];
					$inner_build .= LD.'REL['.$rel_entry.']['.$rel_data['field_name'].']'.$rel_data['marker'].'REL'.RD;
				}
			}

			$tagdata = str_replace(LD.'REL['.$rel_data['field_name'].']'.$rel_data['marker'].'REL'.RD, $inner_build, $tagdata);
		}

		return $tagdata;
	}
}

?>
